﻿using System;
using System.Configuration;
using Nest;

namespace Search.Library
{
    public class ElasticConfiguration
    {
        public const string DefaultIndexName = "default";
        public const string LowerCaseAnalyzerName = "customLowerCase";
        private ElasticClient _client;
        public ElasticConfiguration()
        {
            _client = CreatElasticClient();
        }

        public ElasticClient CreatElasticClient()
        {
            var settings = new ConnectionSettings(new Uri(ConfigurationManager.AppSettings["elasticUrl"]))
                .InferMappingFor<Location>(m => m.IdProperty(p => p.CityId))
                .MapDefaultTypeNames(dictionary => dictionary.Add(typeof(Location), "locations"));
            return new ElasticClient(settings);
        }

        public void SetupMappings()
        {
            var indexExists = _client.IndexExists(new IndexExistsRequest(Indices.Parse(DefaultIndexName)));
            if (indexExists.Exists)
            {
                var type = _client.TypeExists(new TypeExistsDescriptor(Indices.Parse(DefaultIndexName), Types.Type<Location>()));
                if (type.Exists)
                {
                    var mappings = _client.GetMapping<Location>();
                    if (mappings.Mapping == null)
                    {
                        CreateMappings();
                    }
                }
                else
                {
                    CreateMappings();
                }
            }
            else
            {
                CreateIndex();
                CreateMappings();
            }
        }

        private void CreateIndex()
        {
            _client.CreateIndex(DefaultIndexName, descriptor =>
            {
                descriptor.Settings(settingsDescriptor =>
                {
                    settingsDescriptor.Analysis(analysisDescriptor =>
                    {
                        analysisDescriptor.Analyzers(analyzersDescriptor =>
                        {
                            analyzersDescriptor.Custom(LowerCaseAnalyzerName, analyzerDescriptor =>
                            {
                                analyzerDescriptor.Filters("lowercase");
                                analyzerDescriptor.Tokenizer("keyword");
                                return analyzerDescriptor;
                            });
                            return analyzersDescriptor;
                        });
                        return analysisDescriptor;
                    });
                    return settingsDescriptor;
                });
                return descriptor;
            });
        }

        private void CreateMappings()
        {
            _client.Map<Location>(descriptor =>
            {
                descriptor.Index(DefaultIndexName);
                descriptor.Properties(propertiesDescriptor =>
                {
                    propertiesDescriptor.Number(loc => loc.Name(location => location.CityId));
                    propertiesDescriptor.String(loc => loc.Name(location => location.City));
                    propertiesDescriptor.String(loc => loc.Name(location => location.Country));
                    propertiesDescriptor.String(loc => loc.Name(location => location.State));
                    propertiesDescriptor.String(loc => loc.Name(location => location.Type));
                    propertiesDescriptor.String(loc => loc.Name(location => location.Zip)
                        .NotAnalyzed().Analyzer(LowerCaseAnalyzerName));
                    propertiesDescriptor.Number(loc => loc.Name(location => location.Latitude));
                    propertiesDescriptor.Number(loc => loc.Name(location => location.Latitude));
                    propertiesDescriptor.Number(loc => loc.Name(location => location.EstimatedPopulation));
                    propertiesDescriptor.GeoPoint(loc =>
                    {
                        loc.Name(location => location.Coordinates);
                        loc.LatLon();
                        return loc;
                    });
                    return propertiesDescriptor;
                });
                return descriptor;
            });
        }
    }
}
